@@ -39,6 +39,30 @@ static inline bool acvp_match_cipher(cipher_t combination, cipher_t single)
 	return ((combination &~ ~single) == single);
 }
 
+/*
+ * Check that one given length definition is larger than the minimum
+ * and smaller than a given maximum
+ */
+int acvp_req_valid_range_one(unsigned int min, unsigned int max,
+			     unsigned int step, int supported_length);
+
+/*
+ * Check that a supported length definition is larger than the minimum
+ * and smaller than a given maximum
+ */
+int acvp_req_valid_range(unsigned int min, unsigned int max, unsigned int step,
+			 const int supported_lengths[]);
+/*
+ * Check whether a value is in a supported length definition.
+ *
+ * @param val [in] Value to be checked
+ * @param supported_lengths [in] Range domain or list of integers defining the
+ *				 range of allowed parameters.
+ *
+ * return 0 if value is within the range. < 0 on error
+ */
+int acvp_req_in_range(unsigned int val, const int supported_lengths[]);
+
 /*
  * Add a domain specification with min/max/increment to entry at keyword
  * key.
@@ -67,13 +91,17 @@ int acvp_req_algo_int_array_len(struct json_object *entry, const int vals[],
  * Generate the prerequisite entry
  */
 int acvp_req_gen_prereq(const struct def_algo_prereqs *in_prereqs,
-			unsigned int num, struct json_object *entry,
+			unsigned int num,
+			const struct acvp_test_deps *deps,
+			struct json_object *entry,
 			bool publish);
 
 /*
  * Add keyLen array for symmetric ciphers
  */
 int acvp_req_sym_keylen(struct json_object *entry, unsigned int keyflags);
+int acvp_set_sym_keylen(cipher_t keylen[DEF_ALG_MAX_INT],
+			unsigned int keyflags);
 
 /*
  * Add flag for TDES keying option
@@ -104,6 +132,26 @@ int acvp_req_cipher_to_string(struct json_object *entry, cipher_t cipher,
 int acvp_req_cipher_to_array(struct json_object *entry, cipher_t cipher,
 			     cipher_t cipher_type_mask, const char *key);
 
+/*
+ * Convert an internal representation of the cipher reference to an array
+ * of cipher_t entries. I.e. if multiple definitions are found in
+ * the cipher parameter, they are separated into individual entries
+ *
+ * Using the cipher_type_mask, the caller can narrow the search to only
+ * the ciphers with this type mask. It is permissible to use 0 as mask.
+ */
+int acvp_req_cipher_to_intarray(cipher_t cipher,
+				cipher_t cipher_type_mask,
+				cipher_t array[DEF_ALG_MAX_INT]);
+
+/*
+ * Convert an internal representation of the cipher reference to a
+ * comma-delimited string.
+ */
+int acvp_req_cipher_to_stringarray(cipher_t cipher,
+				   cipher_t cipher_type_mask,
+				   char **str);
+
 /*
  * Add request revision number.
  */
@@ -164,6 +212,18 @@ int acvp_extend_string(char *string, size_t stringmaxlen,
  */
 int acvp_get_trailing_number(const char *string, uint32_t *number);
 
+/**
+ * Perform an exact or fuzzy match between two search strings.
+ * @param searchstr needle
+ * @param defstr haystack
+ * @param fuzzy_search If true, perform a substring search, otherwise an
+ *		       exact search is performed.
+ *
+ * @return true for match, false for no match
+ */
+bool acvp_find_match(const char *searchstr, const char *defstr,
+		     bool fuzzy_search);
+
 
 #ifdef __cplusplus
 }
